/*
 * Tag list and selector widget
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma implementation

#include "TagSelector.h"
#include "TagMenu.h"

#include "Environment.h"

#include "DebtagsDocument.h"

#include <gtkmm/eventbox.h>

using namespace std;

template class TagSelector<DebtagsDocument>;

template<class DOC>
void TagSelector<DOC>::do_changed()
{
	_signal_changed.emit();
}

template<class DOC>
void TagSelector<DOC>::do_add_tag(string tag)
{
	_signal_add_tag.emit(tag);
}

template<class DOC>
void TagSelector<DOC>::do_remove_tag(string tag)
{
	_signal_remove_tag.emit(tag);
}

template<class DOC>
bool TagSelector<DOC>::on_titleAdd_event(GdkEvent* e)
{
	if (e->type == GDK_BUTTON_PRESS)
	{
		TagMenu<DOC>* addMenu = manage(new TagMenu<DOC>());
		fprintf(stderr, "PopulateAvailable for TagSelector: %d items\n", items.size());
		addMenu->populateAvailable(doc, items);
		addMenu->signal_selected().connect(
					SigC::slot(*this, &TagSelector<DOC>::on_add_to_selection));
		addMenu->popup(e->button.button, e->button.time);
		return true;
	}
	return false;
}

template<class DOC>
void TagSelector<DOC>::on_add_to_selection(std::string tag)
{
	//printf("Add to sel: %.*s\n", PFSTR(tag));
	unsigned int count_pre = _selected.size();
	_selected += tag;
	if (count_pre != _selected.size())
	{
		updateView();
		do_changed();
	}
}

template<class DOC>
void TagSelector<DOC>::on_del_from_selection(std::string tag)
{
	//printf("Del from sel: %.*s\n", PFSTR(tag));
	unsigned int count_pre = _selected.size();
	_selected -= tag;
	if (count_pre != _selected.size())
	{
		updateView();
		do_changed();
	}
}

template<class DOC>
void TagSelector<DOC>::on_document_changed()
{
	time_t start = time(NULL);
	OpSet<string> allTags = doc.collection().getAllTags();
	_selected = _selected ^ allTags;
	updateView();
	time_t end = time(NULL);
	if (end != start)
		fprintf(stderr, "TagSelector<DOC>::on_document_changed: %d seconds\n", (end-start));
}

template<class DOC>
TagSelector<DOC>::TagSelector(DOC& doc, const std::vector<Debtags::Package*>& items)
	: doc(doc), items(items), table(0)
{
	//set_shadow_type(Gtk::SHADOW_NONE);

	updateView();

	pack_start(tableBox, false, true);
	pack_start(*manage(new Gtk::Label("")), true, true);
	/*
	Gtk::VBox* vbox = manage(new Gtk::VBox());
	vbox->pack_start(tableBox, false, true);
	vbox->pack_start(*manage(new Gtk::Label("CIPS")), true, true);
	add(*vbox);
	*/

	doc.signal_changed().connect(SigC::slot(*this, &TagSelector<DOC>::on_document_changed));
}

template<class DOC>
void TagSelector<DOC>::on_selectedTagList_drag_data_get(
		const Glib::RefPtr<Gdk::DragContext>&, GtkSelectionData* selection_data, guint, guint, string tag)
{
	gtk_selection_data_set(selection_data, selection_data->target, 8, (const guchar*)tag.data(), tag.size());
}

template<class DOC>
void TagSelector<DOC>::updateView()
{
	time_t start = time(NULL);

	if (table)
	{
		tableBox.remove(*table);
		delete table;
	}
	//table.children().clear();

	// Resize the table to fit everything
	//printf("Resized: %dx%d\n", facets+tags+1, 4);
	//table.resize(facets + tags + 1, 4);
	table = new Gtk::Table(_selected.size() + 1, 3);
	tableBox.pack_start(*table);

	// Setup the lists as drag sources
	std::list<Gtk::TargetEntry> listTargets;
	listTargets.push_back(Gtk::TargetEntry("TAG"));
	listTargets.push_back(Gtk::TargetEntry("text/plain"));

	// Create the facet/tag hierarchy
	int row = 0;
    for (OpSet<std::string>::const_iterator i = _selected.begin(); i != _selected.end(); i++)
	{
		Gtk::Button* delTag = manage(new Gtk::Button("-"));
		table->attach(*delTag, 0, 1, row, row + 1, Gtk::SHRINK);
		delTag->signal_clicked().connect(SigC::bind<string>(
					SigC::slot(*this, &TagSelector<DOC>::on_del_from_selection),
					*i));

		Gtk::Label* tag = manage(new Gtk::Label(*i, Gtk::ALIGN_LEFT));
		int px, py;
		tag->get_padding(px, py);
		tag->set_padding(14, py);
		table->attach(*tag, 1, 3, row, row + 1);
		
		++row;
	}

	// Create the title
	//Gtk::Label* title = manage(new Gtk::Label("Selected"));
	////table->attach(*title, 0, 2, 0, 1);
	
	//Gtk::Button* titleAdd = manage(new Gtk::Button("+"));
	//table->attach(*titleAdd, 2, 3, 0, 1, Gtk::SHRINK);
	//titleAdd->add_events(Gdk::BUTTON_PRESS_MASK);
	//titleAdd->signal_event().connect(SigC::slot(*this, &TagSelector<DOC>::on_titleAdd_event));
	//titleAdd->signal_clicked().connect(
			//SigC::slot(*this, &TagcollEditor::on_titleAdd_clicked));

	Gtk::Button* titleAdd = manage(new Gtk::Button("Add"));
	table->attach(*titleAdd, 0, 3, row, row+1, Gtk::SHRINK);
	titleAdd->add_events(Gdk::BUTTON_PRESS_MASK);
	titleAdd->signal_event().connect(SigC::slot(*this, &TagSelector<DOC>::on_titleAdd_event));

	table->show();
	table->show_all_children(true);


	time_t end = time(NULL);
	if (end != start)
		fprintf(stderr, "TagSelector<DOC>::updateList: %d seconds\n", (end-start));
}

// vim:set ts=4 sw=4:
